# -*- coding: utf-8 -*-

"""Console script for gfw_creator."""
import gfw_creator

import sys
import click


@click.command()
@click.argument("lat_0", type=float)
@click.argument("lat_1", type=float)
@click.argument("lon_0", type=float)
@click.argument("lon_1", type=float)
@click.argument("hosing", type=float)
def main(lat_0, lat_1, lon_0, lon_1, hosing):
    ds = gfw_creator.create_homogeneous_hosing(lat_0, lat_1, lon_0, lon_1, hosing)
    ds.to_netcdf("out.nc")
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-

"""Main module."""

import cdo
import xarray as xr

import os
import pkgutil
import subprocess
import sys


def build_cdo_object():
    try:
        return cdo.Cdo()
    except Exception as e:
        print("Python exception:", e)
        print("Unable to use CDO, this won't work, sorry...")
        print("You may consider running >> module load cdo << and trying to run again")
        sys.exit(1)

def _get_template_file():
    """
    Creates a local copy of the gfw_atmo template file.

    Returns
    -------
    str :
        Path to the file that was generated
    """
    template_file_path = "./gfw_atmo_template.nc"
    template_file_data = pkgutil.get_data("gfw_creator", "data/gfw_atmo_template.nc")
    with open(template_file_path, "wb") as template_file:
        template_file.write(template_file_data)
    return os.path.abspath(template_file_path)


def _load_template_file():
    """
    Loads the template file to be modified.

    Returns
    -------
    xr.Dataset :
        The template dataset, has 0 everywhere with 366 day timesteps. This is
        on the standard ECHAM6 T63 grid.
    """
    df = xr.open_dataset(_get_template_file())
    df.coords["lon"] = (df.coords["lon"] + 180) % 360 - 180
    df = df.sortby(df.lon)
    return df


def _lat_lon_area(lat_0, lat_1, lon_0, lon_1):
    """
    Figures out the area of a lat/lon box for distributing a flux across a
    total area
    """
    CDO = build_cdo_object()
    ds = _get_template_file()
    selected_ds = CDO.fldsum(
        input=f"-sellonlatbox,{lon_0},{lon_1},{lat_0},{lat_1} -selname,cell_area {ds}",
        returnXDataset=True,
    )
    return selected_ds.cell_area.data.squeeze()


def create_homogeneous_hosing(lat_0, lat_1, lon_0, lon_1, hosing_strength):
    """
    Creates a homogeneous hosing field
    """
    CDO = build_cdo_object()
    ds = _get_template_file()
    # Notes:
    # ------
    # `hosing_strength` is given in Sv. Convert this to m3/s:
    #
    # 1 Sv = 10**6 m3/s
    hosing_strength *= 1.0e6
    # The gfw_atmo file must be given in m/s, so we divide by the area for
    # conversion m3/s --> m/s :
    hosing_per_area = hosing_strength / _lat_lon_area(lat_0, lat_1, lon_0, lon_1)
    # Now that the hosing strength has been determined, set it in the NetCDF
    # file:
    gfw_atmo_file = CDO.setclonlatbox(
        f"{hosing_per_area},{lon_0},{lon_1},{lat_0},{lat_1}",
        input=ds,
        returnXDataset=True,
    )
    return gfw_atmo_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "Click>=6.0",
    "xarray",
    "cdo",
    "netcdf4",
]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest",
]

setup(
    author="Paul Gierz",
    author_email="pgierz@awi.de",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
    ],
    description="GFW Creator allows you to generate files for the gfw_atmo switch in ECHAM6/JSBACH of AWI-ESM",
    entry_points={
        "console_scripts": ["gfw_creator=gfw_creator.cli:main",],
        "esm_tools.plugins": [
            "create_hosing_files=gfw_creator.esm_tools_plugin:create_hosing_files",
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="gfw_creator",
    name="gfw_creator",
    packages=find_packages(include=["gfw_creator"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/pgierz/gfw_creator",
    version="0.1.0",
    zip_safe=False,
)



from . import gfw_creator

import sys

from loguru import logger


def create_hosing_files(config):
    if config["general"]["setup_name"] == "awiesm":
        if config["fesom"].get("do_hosing"):
            logger.info("Creating hosing files...")
            hosing_type = config["fesom"].get("hosing_type")
            if hosing_type == "homogeneous":
                lat_0 = config["fesom"]["hosing_lat0"]
                lat_1 = config["fesom"]["hosing_lat1"]
                lon_0 = config["fesom"]["hosing_lon0"]
                lon_1 = config["fesom"]["hosing_lon1"]
                hosing_strength = config["fesom"]["hosing_strength"]
                gfw_atmo_file = gfw_creator.create_homogeneous_hosing(
                    lat_0, lat_1, lon_0, lon_1, hosing_strength
                )
                gfw_atmo_file.to_netcdf(
                    config["fesom"]["thisrun_forcing_dir"] + "/gfw_atmo.nc"
                )
                logger.info(
                    f"Wrote {config['fesom']['thisrun_forcing_dir'] + '/gfw_atmo.nc'} for use in the simulation!"
                )
                config["oasis3mct"].setdefault("coupling_input_fields", {})
                config["oasis3mct"]["coupling_input_fields"]["gfw_atmo"] = {
                    "freq": 86400,
                    "field_filepath": "gfw_atmo.nc",
                }
                config["fesom"].setdefault("forcing_files", {})
                config["fesom"].setdefault("forcing_sources", {})
                config["fesom"].setdefault("forcing_in_work", {})

                config["fesom"]["forcing_files"]["gfw_atmo"] = "gfw_atmo"
                config["fesom"]["forcing_sources"]["gfw_atmo"] = (
                    config["fesom"]["thisrun_forcing_dir"] + "/gfw_atmo.nc"
                )
                config["fesom"]["forcing_in_work"]["gfw_atmo"] = "gfw_atmo.nc"
                return config
            elif hosing_type == "from_file":
                logger.error("Sorry, hosing from a file has no yet been implemented!")
                sys.exit(1)
                # TODO(PG): Later...
                # hosing_file = config["fesom"]["hosing_file"]
            else:
                logger.error(
                    "Sorry, you must select between hosing_type: 'homogeneous' or hosing_type: 'from_file' in your fesom configuration!"
                )
                sys.exit(1)

    return config
